// https://www.te.com/commerce/DocumentDelivery/DDEController?Action=showdoc&DocId=Data+Sheet%7FMS5611-01BA03%7FB3%7Fpdf%7FEnglish%7FENG_DS_MS5611-01BA03_B3.pdf%7FCAT-BLPS0036
// https://github.com/adafruit/TinyWireM
// https://github.com/orgua/OneWireHub
// ATtiny85 8MHz internal BOD 4.3V

#include <avr/wdt.h>
#include "TinyWireM.h" 
#include "OneWireHub.h"
#include "DS18B20.h" 

//#define HIRES
// if defined master can extract pressure with original MS5611 resolution:
// float P_ds = ((scratchpad[1] << 8) | scratchpad[0]) / 16.0f; // stanrdard resolution 0.0625 kPa
// uint32_t P_hr = (uint32_t)P_ds * 1000 + ((uint16_t)scratchpad[3] << 8) | scratchpad[2]; // 1 Pa
// float P_hr = (uint8_t)P_ds + (float)(((uint16_t)scratchpad[3] << 8) | scratchpad[2]) / 1000.0f; // 0.001 KPa

uint16_t C1, C2, C3, C4, C5, C6;
OneWireHub hub = OneWireHub(1);
DS18B20 ds18b20 = DS18B20(DS18B20::family_code, 'm', 's', 0x56, 0x11, 0x00, 0x03); // 48bit ROM address

void ms5611_request(uint8_t n)
{
  TinyWireM.beginTransmission(0x77); 
  TinyWireM.requestFrom(0x77, n); 
  while(!TinyWireM.available());
}

void ms5611_write_8(uint8_t d) 
{
  TinyWireM.beginTransmission(0x77); 
  TinyWireM.write(d); 
  TinyWireM.endTransmission();
}

uint16_t ms5611_read_16 (uint8_t addr)
{
  ms5611_write_8(addr);
  ms5611_request(2);
  uint16_t b0 = TinyWireM.read();
  uint8_t b1 = TinyWireM.read();
  TinyWireM.endTransmission();
  return b0 << 8 | b1;
}

uint32_t ms5611_read_32 ()
{
  ms5611_write_8(0);
  ms5611_request(3);
  uint32_t b1 = TinyWireM.read();
  uint16_t b2 = TinyWireM.read();
  uint8_t b3 = TinyWireM.read();
  TinyWireM.endTransmission();
  return (b1 << 16) | (b2 << 8) | b3;
}

void convert_t ()
{
  digitalWrite(3, 1);

  ms5611_write_8(0x48);         // convert D1 osr 4096
  delay(10);                    // 9.04ms max
  uint32_t D1 = ms5611_read_32();

  ms5611_write_8(0x58);         // convert D2 osr 4096
  delay(10);                    // 9.04ms max
  uint32_t D2 = ms5611_read_32();

  int32_t dT = D2 - (uint32_t)C5 * 0x100;
  int64_t OFF = (int64_t)C2 * 0x10000 + (int64_t)C4 * dT / 0x80;
  int64_t SENS = (int64_t)C1 * 0x8000 + (int64_t)C3 * dT / 0x100;
  int32_t TEMP = 2000 + ((int64_t)dT * C6) / 0x800000;
  int64_t OFF2 = 0;
  int64_t SENS2 = 0;
  if (TEMP < 2000)
  {
    OFF2 = 5 * ((TEMP - 2000) * (TEMP - 2000)) / 2;
    SENS2 = 5 * ((TEMP - 2000) * (TEMP - 2000)) / 4;
    if (TEMP < -1500)
    {
      OFF2 += 7 * ((TEMP + 1500) * (TEMP + 1500));
      SENS2 += 11 * ((TEMP + 1500) * (TEMP + 1500)) / 2;
    }
  }
  OFF -= OFF2;
  SENS -= SENS2;

  int32_t P = (D1 * SENS / 0x200000 - OFF) / 0x8000;  // compensated pressure, Pa
#ifdef HIRES
  uint16_t Pf = P % 1000;
  ds18b20.scratchpad[2] = Pf & 0xff;
  ds18b20.scratchpad[3] = Pf >> 8;
#endif
  ds18b20.setTemperatureRaw(P * 16 / 1000); // KPa*16 (0.0625 resolution)
}

void read_scratchpad () 
{
  digitalWrite(3, 0);
}

void setup()
{
  wdt_enable(WDTO_60MS);
  TinyWireM.begin();

  ms5611_write_8(0x1e);         // reset
  delay(3);                     // 2.8ms, fig.5
  C1 = ms5611_read_16(0xa2);    // 6*16 calibration data
  C2 = ms5611_read_16(0xa4);
  C3 = ms5611_read_16(0xa6);
  C4 = ms5611_read_16(0xa8);
  C5 = ms5611_read_16(0xaa);
  C6 = ms5611_read_16(0xac);

  pinMode(3, OUTPUT);
  wdt_reset();
  convert_t();           // 1st measure & led on
  read_scratchpad();     // led off
  ds18b20.convert_t_ptr = &convert_t;
  ds18b20.read_scratchpad_ptr = &read_scratchpad;
  hub.attach(ds18b20);
}

void loop()
{ 
  wdt_reset();
  hub.poll();
}
